@@ -16,8 +16,6 @@
 #include <afsconfig.h>
 #include "afs/param.h"
 
-RCSID
-    ("$Header: /cvs/openafs/src/afs/AIX/osi_inode.c,v 1.8 2003/07/15 23:14:17 shadow Exp $");
 
 #include "afs/sysincludes.h"	/* Standard vendor system headers */
 #include "afsincludes.h"	/* Afs-based standard headers */
@@ -144,7 +142,7 @@ devtovfs(dev_t dev)
 
     a.dev = dev;
     a.ans = NULL;
-    vfs_search(devtovfs_func, &a);
+    vfs_search(devtovfs_func, (caddr_t) &a);
     return a.ans;
 }
 
@@ -236,13 +234,13 @@ igetinode(dev, vfsp, inode, vpp, perror)
 	ip = 0;
 	goto out;
     }
-    IREAD_UNLOCK(ip);
     if (vpp) {
 	if (nvfsp)
 	    *vpp = ip->i_gnode.gn_vnode;
 	else
 	    setuerror(iptovp(vfsp, ip, vpp));
     }
+    IREAD_UNLOCK(ip);
   out:
     return ip;
 }
@@ -262,7 +260,6 @@ SYSENT(icreate, (dev, near_inode, param1, param2, param3, param4),)
     struct inode *ip, *newip, *pip;
     register int err, rval1, rc = 0;
     struct vnode *vp = NULL;
-    extern struct vfs *rootvfs;
     register struct vfs *vfsp;
     struct vfs *nvfsp = NULL;
     char error;
