@@ -14,8 +14,6 @@
 #include <afsconfig.h>
 #include "afs/param.h"
 
-RCSID
-    ("$Header: /cvs/openafs/src/rx/rx_kcommon.c,v 1.44.2.4 2005/07/27 18:17:46 shadow Exp $");
 
 #include "rx/rx_kcommon.h"
 
@@ -26,8 +24,8 @@ RCSID
 #include "afsint.h"
 
 #ifndef RXK_LISTENER_ENV
-int (*rxk_PacketArrivalProc) (register struct rx_packet * ahandle, register struct sockaddr_in * afrom, char *arock, afs_int32 asize);	/* set to packet allocation procedure */
-int (*rxk_GetPacketProc) (char **ahandle, int asize);
+int (*rxk_PacketArrivalProc) (struct rx_packet * ahandle, struct sockaddr_in * afrom, struct socket *arock, afs_int32 asize);	/* set to packet allocation procedure */
+int (*rxk_GetPacketProc) (struct rx_packet **ahandle, int asize);
 #endif
 
 osi_socket *rxk_NewSocketHost(afs_uint32 ahost, short aport);
@@ -45,6 +43,11 @@ static int myNetMTUs[ADDRSPERSITE];
 static int numMyNetAddrs = 0;
 #endif
 
+#if defined(AFS_DARWIN80_ENV)
+#define sobind sock_bind
+#define soclose sock_close
+#endif
+
 /* add a port to the monitored list, port # is in network order */
 static int
 rxk_AddPort(u_short aport, char *arock)
@@ -119,6 +122,7 @@ rxi_GetUDPSocket(u_short port)
     return rxi_GetHostUDPSocket(htonl(INADDR_ANY), port);
 }
 
+#if !defined(AFS_LINUX26_ENV)
 void
 osi_Panic(msg, a1, a2, a3)
      char *msg;
@@ -127,7 +131,11 @@ osi_Panic(msg, a1, a2, a3)
 	msg = "Unknown AFS panic";
 
     printf(msg, a1, a2, a3);
+#ifdef AFS_LINUX20_ENV
+    * ((char *) 0) = 0; 
+#else
     panic(msg);
+#endif
 }
 
 /*
@@ -252,12 +260,19 @@ osi_AssertFailK(const char *expr, const char *file, int line)
 
     osi_Panic(buf);
 }
+#else
+void
+osi_AssertFailK(const char *expr, const char *file, int line)
+{
+    printk(KERN_CRIT "assertion failed: %s, file: %s, line: %d\n", expr, file, line);
+}
+#endif
 
 #ifndef UKERNEL
 /* This is the server process request loop. Kernel server
  * processes never become listener threads */
-void
-rx_ServerProc(void)
+void *
+rx_ServerProc(void *unused)
 {
     int threadID;
 
@@ -274,15 +289,17 @@ rx_ServerProc(void)
 #ifdef RX_ENABLE_LOCKS
     AFS_GLOCK();
 #endif /* RX_ENABLE_LOCKS */
+
+    return NULL;
 }
 #endif /* !UKERNEL */
 
 #ifndef RXK_LISTENER_ENV
 /* asize includes the Rx header */
 static int
-MyPacketProc(char **ahandle, int asize)
+MyPacketProc(struct rx_packet **ahandle, int asize)
 {
-    register struct rx_packet *tp;
+    struct rx_packet *tp;
 
     /* If this is larger than we expected, increase rx_maxReceiveDataSize */
     /* If we can't scrounge enough cbufs, then we have to drop the packet,
@@ -312,27 +329,28 @@ MyPacketProc(char **ahandle, int asize)
 	rx_stats.bogusPacketOnRead++;
 	MUTEX_EXIT(&rx_stats_mutex);
 	/* I DON"T LIKE THIS PRINTF -- PRINTFS MAKE THINGS VERY VERY SLOOWWW */
-	printf("rx: packet dropped: bad ulen=%d\n", asize);
+	dpf(("rx: packet dropped: bad ulen=%d\n", asize));
 	tp = NULL;
     }
 
     if (!tp)
 	return -1;
     /* otherwise we have a packet, set appropriate values */
-    *ahandle = (char *)tp;
+    *ahandle = tp;
     return 0;
 }
 
 static int
-MyArrivalProc(register struct rx_packet *ahandle,
-	      register struct sockaddr_in *afrom, char *arock,
+MyArrivalProc(struct rx_packet *ahandle,
+	      struct sockaddr_in *afrom,
+	      struct socket *arock,
 	      afs_int32 asize)
 {
     /* handle basic rx packet */
     ahandle->length = asize - RX_HEADER_SIZE;
     rxi_DecodePacketHeader(ahandle);
     ahandle =
-	rxi_ReceivePacket(ahandle, (struct socket *)arock,
+	rxi_ReceivePacket(ahandle, arock,
 			  afrom->sin_addr.s_addr, afrom->sin_port, NULL,
 			  NULL);
 
@@ -369,7 +387,7 @@ rxi_InitPeerParams(register struct rx_peer *pp)
     if (i == -1) {
 	pp->timeout.sec = 3;
 	/* pp->timeout.usec = 0; */
-	pp->ifMTU = RX_REMOTE_PACKET_SIZE;
+	pp->ifMTU = MIN(RX_REMOTE_PACKET_SIZE, rx_MyMaxSendSize);
     } else {
 	pp->timeout.sec = 2;
 	/* pp->timeout.usec = 0; */
@@ -385,10 +403,14 @@ rxi_InitPeerParams(register struct rx_peer *pp)
 		pp->ifMTU = rxmtu;
 	}
     } else {			/* couldn't find the interface, so assume the worst */
-	pp->ifMTU = RX_REMOTE_PACKET_SIZE;
+	pp->ifMTU = MIN(RX_REMOTE_PACKET_SIZE, rx_MyMaxSendSize);
     }
 #else /* AFS_USERSPACE_IP_ADDR */
+#ifdef AFS_DARWIN80_ENV
+    ifnet_t ifn;
+#else
     struct ifnet *ifn;
+#endif
 
 #if !defined(AFS_SGI62_ENV)
     if (numMyNetAddrs == 0)
@@ -401,7 +423,7 @@ rxi_InitPeerParams(register struct rx_peer *pp)
 	/* pp->timeout.usec = 0; */
 	pp->ifMTU = MIN(RX_MAX_PACKET_SIZE, rx_MyMaxSendSize);
 #ifdef IFF_POINTOPOINT
-	if (ifn->if_flags & IFF_POINTOPOINT) {
+	if (ifnet_flags(ifn) & IFF_POINTOPOINT) {
 	    /* wish we knew the bit rate and the chunk size, sigh. */
 	    pp->timeout.sec = 4;
 	    pp->ifMTU = RX_PP_PACKET_SIZE;
@@ -409,15 +431,15 @@ rxi_InitPeerParams(register struct rx_peer *pp)
 #endif /* IFF_POINTOPOINT */
 	/* Diminish the packet size to one based on the MTU given by
 	 * the interface. */
-	if (ifn->if_mtu > (RX_IPUDP_SIZE + RX_HEADER_SIZE)) {
-	    rxmtu = ifn->if_mtu - RX_IPUDP_SIZE;
+	if (ifnet_mtu(ifn) > (RX_IPUDP_SIZE + RX_HEADER_SIZE)) {
+	    rxmtu = ifnet_mtu(ifn) - RX_IPUDP_SIZE;
 	    if (rxmtu < pp->ifMTU)
 		pp->ifMTU = rxmtu;
 	}
     } else {			/* couldn't find the interface, so assume the worst */
 	pp->timeout.sec = 3;
 	/* pp->timeout.usec = 0; */
-	pp->ifMTU = RX_REMOTE_PACKET_SIZE;
+	pp->ifMTU = MIN(RX_REMOTE_PACKET_SIZE, rx_MyMaxSendSize);
     }
 #endif /* else AFS_USERSPACE_IP_ADDR */
 #else /* AFS_SUN5_ENV */
@@ -426,7 +448,7 @@ rxi_InitPeerParams(register struct rx_peer *pp)
     if (mtu <= 0) {
 	pp->timeout.sec = 3;
 	/* pp->timeout.usec = 0; */
-	pp->ifMTU = RX_REMOTE_PACKET_SIZE;
+	pp->ifMTU = MIN(RX_REMOTE_PACKET_SIZE, rx_MyMaxSendSize);
     } else {
 	pp->timeout.sec = 2;
 	/* pp->timeout.usec = 0; */
@@ -442,7 +464,7 @@ rxi_InitPeerParams(register struct rx_peer *pp)
 		pp->ifMTU = rxmtu;
 	}
     } else {			/* couldn't find the interface, so assume the worst */
-	pp->ifMTU = RX_REMOTE_PACKET_SIZE;
+	pp->ifMTU = MIN(RX_REMOTE_PACKET_SIZE,rx_MyMaxSendSize);
     }
 #endif /* AFS_SUN5_ENV */
 #else /* ADAPT_MTU */
@@ -455,7 +477,7 @@ rxi_InitPeerParams(register struct rx_peer *pp)
     pp->natMTU = MIN(pp->ifMTU, OLD_MAX_PACKET_SIZE);
     pp->ifDgramPackets =
 	MIN(rxi_nDgramPackets,
-	    rxi_AdjustDgramPackets(RX_MAX_FRAGS, pp->ifMTU));
+	    rxi_AdjustDgramPackets(rxi_nSendFrags, pp->ifMTU));
     pp->maxDgramPackets = 1;
 
     /* Initialize slow start parameters */
@@ -491,7 +513,7 @@ shutdown_rxkernel(void)
 	    rxk_shutdownPorts();
 	    return;
 	}
-    printf("shutdown_rxkernel: no udp proto");
+    dpf(("shutdown_rxkernel: no udp proto"));
 }
 #endif /* !AIX && !SUN && !NCR  && !UKERNEL */
 
@@ -502,7 +524,7 @@ shutdown_rxkernel(void)
 int
 rxi_GetcbiInfo(void)
 {
-    int i, j, different = 0;
+    int i, j, different = 0, num = ADDRSPERSITE;
     int rxmtu, maxmtu;
     afs_uint32 ifinaddr;
     afs_uint32 addrs[ADDRSPERSITE];
@@ -511,7 +533,9 @@ rxi_GetcbiInfo(void)
     memset((void *)addrs, 0, sizeof(addrs));
     memset((void *)mtus, 0, sizeof(mtus));
 
-    for (i = 0; i < afs_cb_interface.numberOfInterfaces; i++) {
+    if (afs_cb_interface.numberOfInterfaces < num)
+	num = afs_cb_interface.numberOfInterfaces;
+    for (i = 0; i < num; i++) {
 	if (!afs_cb_interface.mtu[i])
 	    afs_cb_interface.mtu[i] = htonl(1500);
 	rxmtu = (ntohl(afs_cb_interface.mtu[i]) - RX_IPUDP_SIZE);
@@ -611,16 +635,62 @@ rxi_GetIFInfo(void)
     int i = 0;
     int different = 0;
 
-    register struct ifnet *ifn;
     register int rxmtu, maxmtu;
     afs_uint32 addrs[ADDRSPERSITE];
     int mtus[ADDRSPERSITE];
-    struct ifaddr *ifad;	/* ifnet points to a if_addrlist of ifaddrs */
     afs_uint32 ifinaddr;
+#if defined(AFS_DARWIN80_ENV)
+    errno_t t;
+    unsigned int count;
+    int cnt=0, m, j;
+    ifaddr_t *ifads;
+    ifnet_t *ifn;
+    struct sockaddr sout;
+    struct sockaddr_in *sin;
+    struct in_addr pin;
+#else
+    struct ifaddr *ifad;	/* ifnet points to a if_addrlist of ifaddrs */
+    register struct ifnet *ifn;
+#endif
 
     memset(addrs, 0, sizeof(addrs));
     memset(mtus, 0, sizeof(mtus));
 
+#if defined(AFS_DARWIN80_ENV)
+    if (!ifnet_list_get(AF_INET, &ifn, &count)) {
+	for (m = 0; m < count; m++) {
+	    if (!ifnet_get_address_list(ifn[m], &ifads)) {
+		for (j = 0; ifads[j] != NULL && cnt < ADDRSPERSITE; j++) {
+		    if ((t = ifaddr_address(ifads[j], &sout, sizeof(struct sockaddr))) == 0) {
+			sin = (struct sockaddr_in *)&sout;
+			rxmtu = ifnet_mtu(ifaddr_ifnet(ifads[j])) - RX_IPUDP_SIZE;
+			ifinaddr = ntohl(sin->sin_addr.s_addr);
+			if (myNetAddrs[i] != ifinaddr) {
+			    different++;
+			}
+			mtus[i] = rxmtu;
+			rxmtu = rxi_AdjustIfMTU(rxmtu);
+			maxmtu =
+			    rxmtu * rxi_nRecvFrags +
+			    ((rxi_nRecvFrags - 1) * UDP_HDR_SIZE);
+			maxmtu = rxi_AdjustMaxMTU(rxmtu, maxmtu);
+			addrs[i++] = ifinaddr;
+			if ((ifinaddr != 0x7f000001) && 
+			    (maxmtu > rx_maxReceiveSize)) {
+			    rx_maxReceiveSize = 
+				MIN(RX_MAX_PACKET_SIZE, maxmtu);
+			    rx_maxReceiveSize =
+				MIN(rx_maxReceiveSize, rx_maxReceiveSizeUser);
+			}
+			cnt++;
+		    }
+		}
+		ifnet_free_address_list(ifads);
+	    }
+	}
+	ifnet_list_free(ifn);
+    }
+#else
 #if defined(AFS_DARWIN_ENV) || defined(AFS_FBSD_ENV)
     TAILQ_FOREACH(ifn, &ifnet, if_link) {
 	if (i >= ADDRSPERSITE)
@@ -666,6 +736,7 @@ rxi_GetIFInfo(void)
 	    }
 	}
     }
+#endif
 
     rx_maxJumboRecvSize =
 	RX_HEADER_SIZE + rxi_nDgramPackets * RX_JUMBOBUFFERSIZE +
@@ -673,10 +744,10 @@ rxi_GetIFInfo(void)
     rx_maxJumboRecvSize = MAX(rx_maxJumboRecvSize, rx_maxReceiveSize);
 
     if (different) {
-	int j;
-	for (j = 0; j < i; j++) {
-	    myNetMTUs[j] = mtus[j];
-	    myNetAddrs[j] = addrs[j];
+	int l;
+	for (l = 0; l < i; l++) {
+	    myNetMTUs[l] = mtus[l];
+	    myNetAddrs[l] = addrs[l];
 	}
     }
     return different;
@@ -684,19 +755,41 @@ rxi_GetIFInfo(void)
 
 #if defined(AFS_DARWIN60_ENV) || defined(AFS_XBSD_ENV)
 /* Returns ifnet which best matches address */
+#ifdef AFS_DARWIN80_ENV
+ifnet_t
+#else
 struct ifnet *
+#endif
 rxi_FindIfnet(afs_uint32 addr, afs_uint32 * maskp)
 {
-    struct sockaddr_in s;
+    struct sockaddr_in s, sr;
+#ifdef AFS_DARWIN80_ENV
+    ifaddr_t ifad;
+#else
     struct ifaddr *ifad;
+#endif
 
     s.sin_family = AF_INET;
     s.sin_addr.s_addr = addr;
+#ifdef AFS_DARWIN80_ENV
+    ifad = ifaddr_withnet((struct sockaddr *)&s);
+#elif AFS_OBSD46_ENV
+    ifad = ifa_ifwithnet((struct sockaddr *)&s, 0);
+#else
     ifad = ifa_ifwithnet((struct sockaddr *)&s);
+#endif
 
+#ifdef AFS_DARWIN80_ENV
+    if (ifad && maskp) {
+	ifaddr_netmask(ifad, (struct sockaddr *)&sr, sizeof(sr));
+	*maskp = sr.sin_addr.s_addr;
+    }
+    return (ifad ? ifaddr_ifnet(ifad) : NULL);
+#else
     if (ifad && maskp)
 	*maskp = ((struct sockaddr_in *)ifad->ifa_netmask)->sin_addr.s_addr;
     return (ifad ? ifad->ifa_ifp : NULL);
+#endif
 }
 
 #else /* DARWIN60 || XBSD */
@@ -761,7 +854,11 @@ osi_socket *
 rxk_NewSocketHost(afs_uint32 ahost, short aport)
 {
     register afs_int32 code;
+#ifdef AFS_DARWIN80_ENV
+    socket_t newSocket;
+#else
     struct socket *newSocket;
+#endif
 #if (!defined(AFS_HPUX1122_ENV) && !defined(AFS_FBSD50_ENV))
     struct mbuf *nam;
 #endif
@@ -810,6 +907,8 @@ rxk_NewSocketHost(afs_uint32 ahost, short aport)
 		    afs_osi_credp, curthread);
 #elif defined(AFS_FBSD40_ENV)
     code = socreate(AF_INET, &newSocket, SOCK_DGRAM, IPPROTO_UDP, curproc);
+#elif defined(AFS_DARWIN80_ENV)
+    code = sock_socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP, NULL, NULL, &newSocket);
 #else
     code = socreate(AF_INET, &newSocket, SOCK_DGRAM, 0);
 #endif /* AFS_HPUX102_ENV */
@@ -845,12 +944,30 @@ rxk_NewSocketHost(afs_uint32 ahost, short aport)
 
     freeb(bindnam);
 #else /* AFS_HPUX110_ENV */
+#if defined(AFS_DARWIN80_ENV)
+    { 
+       int buflen = 50000;
+       int i,code2;
+       for (i=0;i<2;i++) {
+           code = sock_setsockopt(newSocket, SOL_SOCKET, SO_SNDBUF,
+                                  &buflen, sizeof(buflen));
+           code2 = sock_setsockopt(newSocket, SOL_SOCKET, SO_RCVBUF,
+                                  &buflen, sizeof(buflen));
+           if (!code && !code2)
+               break;
+           if (i == 2)
+	      osi_Panic("osi_NewSocket: last attempt to reserve 32K failed!\n");
+           buflen = 32766;
+       }
+    }
+#else
     code = soreserve(newSocket, 50000, 50000);
     if (code) {
 	code = soreserve(newSocket, 32766, 32766);
 	if (code)
 	    osi_Panic("osi_NewSocket: last attempt to reserve 32K failed!\n");
     }
+#endif
 #if defined(AFS_DARWIN_ENV) || defined(AFS_FBSD_ENV)
 #if defined(AFS_FBSD50_ENV)
     code = sobind(newSocket, (struct sockaddr *)&myaddr, curthread);
@@ -860,7 +977,7 @@ rxk_NewSocketHost(afs_uint32 ahost, short aport)
     code = sobind(newSocket, (struct sockaddr *)&myaddr);
 #endif
     if (code) {
-	printf("sobind fails (%d)\n", (int)code);
+	dpf(("sobind fails (%d)\n", (int)code));
 	soclose(newSocket);
 	AFS_GLOCK();
 	goto bad;
@@ -879,15 +996,17 @@ rxk_NewSocketHost(afs_uint32 ahost, short aport)
     }
     nam->m_len = sizeof(myaddr);
     memcpy(mtod(nam, caddr_t), &myaddr, sizeof(myaddr));
-#ifdef AFS_SGI65_ENV
+#if defined(AFS_SGI65_ENV)
     BHV_PDATA(&bhv) = (void *)newSocket;
     code = sobind(&bhv, nam);
     m_freem(nam);
+#elif defined(AFS_OBSD44_ENV)
+    code = sobind(newSocket, nam, osi_curproc());
 #else
     code = sobind(newSocket, nam);
 #endif
     if (code) {
-	printf("sobind fails (%d)\n", (int)code);
+	dpf(("sobind fails (%d)\n", (int)code));
 	soclose(newSocket);
 #ifndef AFS_SGI65_ENV
 	m_freem(nam);
@@ -977,8 +1096,12 @@ afs_rxevent_daemon(void)
 	if (afs_termState == AFSOP_STOP_RXEVENT) {
 #ifdef RXK_LISTENER_ENV
 	    afs_termState = AFSOP_STOP_RXK_LISTENER;
+#else
+#ifdef AFS_SUN510_ENV
+	    afs_termState = AFSOP_STOP_NETIF;
 #else
 	    afs_termState = AFSOP_STOP_COMPLETE;
+#endif
 #endif
 	    osi_rxWakeup(&afs_termState);
 	    return;
@@ -1046,9 +1169,7 @@ rxk_ReadPacket(osi_socket so, struct rx_packet *p, int *host, int *port)
     if (!code) {
 	p->length = nbytes - RX_HEADER_SIZE;;
 	if ((nbytes > tlen) || (p->length & 0x8000)) {	/* Bogus packet */
-	    if (nbytes > 0)
-		rxi_MorePackets(rx_initSendWindow);
-	    else {
+	    if (nbytes <= 0) {
 		MUTEX_ENTER(&rx_stats_mutex);
 		rx_stats.bogusPacketOnRead++;
 		rx_stats.bogusHost = from.sin_addr.s_addr;
@@ -1093,6 +1214,9 @@ rxk_ReadPacket(osi_socket so, struct rx_packet *p, int *host, int *port)
  * OS's socket receive routine returns as a result of a signal.
  */
 int rxk_ListenerPid;		/* Used to signal process to wakeup at shutdown */
+#ifdef AFS_LINUX20_ENV
+struct task_struct *rxk_ListenerTask;
+#endif
 
 #ifdef AFS_SUN5_ENV
 /*
@@ -1122,6 +1246,7 @@ rxk_Listener(void)
 
 #ifdef AFS_LINUX20_ENV
     rxk_ListenerPid = current->pid;
+    rxk_ListenerTask = current;
 #endif
 #ifdef AFS_SUN5_ENV
     rxk_ListenerPid = 1;	/* No PID, just a flag that we're alive */
@@ -1129,13 +1254,14 @@ rxk_Listener(void)
 #ifdef AFS_XBSD_ENV
     rxk_ListenerPid = curproc->p_pid;
 #endif /* AFS_FBSD_ENV */
-#if defined(AFS_DARWIN_ENV)
+#ifdef AFS_DARWIN80_ENV
+    rxk_ListenerPid = proc_selfpid();
+#elif defined(AFS_DARWIN_ENV)
     rxk_ListenerPid = current_proc()->p_pid;
 #endif
 #if defined(RX_ENABLE_LOCKS) && !defined(AFS_SUN5_ENV)
     AFS_GUNLOCK();
 #endif /* RX_ENABLE_LOCKS && !AFS_SUN5_ENV */
-
     while (afs_termState != AFSOP_STOP_RXK_LISTENER) {
 	if (rxp) {
 	    rxi_RestoreDataBufs(rxp);
@@ -1153,11 +1279,19 @@ rxk_Listener(void)
     AFS_GLOCK();
 #endif /* RX_ENABLE_LOCKS */
     if (afs_termState == AFSOP_STOP_RXK_LISTENER) {
+#ifdef AFS_SUN510_ENV
+	afs_termState = AFSOP_STOP_NETIF;
+#else
 	afs_termState = AFSOP_STOP_COMPLETE;
+#endif
 	osi_rxWakeup(&afs_termState);
     }
     rxk_ListenerPid = 0;
-#if defined(AFS_LINUX22_ENV) || defined(AFS_SUN5_ENV)
+#ifdef AFS_LINUX20_ENV
+    rxk_ListenerTask = 0;
+    osi_rxWakeup(&rxk_ListenerTask);
+#endif
+#if defined(AFS_SUN5_ENV)
     osi_rxWakeup(&rxk_ListenerPid);
 #endif
 #ifdef AFS_SUN5_ENV
